/*
 * ms5637.c
 *
 *  Created on: Aug 12, 2014
 *      Author: JMBrinkhus
 */

#include "ms5637.h"
#include "xiic_l.h"
#include "sleep.h"
#include "math.h"
#include "xparameters.h"

// MS5637 Global Variables
u32 					ms5637_axi_address;
u32 					ms5637_prom_coeffs[7];
ms5637_resolution_osr	ms5637_resolution;

/*********************************************************************
 *
 * Function:    ms5637_init
 *
 * Description:	Initialize the axi address of the i2c core and the
 * 				sensor's over sampling rate to 8192.
 *
 * Parameters:	u32 axi_address -
 * 					Address of Xilinx AXI IIC Peripheral
 *
 * Returns:		void
 *
 *********************************************************************/
void ms5637_init(u32 axi_address){

	ms5637_axi_address = axi_address;
	ms5637_resolution = ms5637_resolution_osr_8192;

	return;

}

/*********************************************************************
 *
 * Function:    ms5637_reset
 *
 * Description:	Send I2C Reset command to MS5637 and wait 100ms
 *
 * Parameters:	void
 *
 * Returns:		Enumerated type ms5637_status with possible values
 * 				ms5637_status_ok or ms5637_status_i2c_transfer_error
 *
 *********************************************************************/
ms5637_status ms5637_reset(void){

	char tx_buf[1];
	int byte_count=0;

	tx_buf[0] = MS5637_I2C_CMD_RESET;
	byte_count = XIic_Send(ms5637_axi_address, MS5637_I2C_ADDR, (u8*)tx_buf, 1, XIIC_STOP);
	if(byte_count!=1){
		return ms5637_status_i2c_transfer_error;
	}
	//Device requires time to complete reset operation
	usleep(100000);

	return ms5637_status_ok;

}

/*********************************************************************
 *
 * Function:    ms5637_read_prom
 *
 * Description:	Read factory calibrated coefficients from PROM
 *
 * Parameters:	void
 *
 * Returns:		Enumerated type ms5637_status with possible values
 * 				ms5637_status_ok or ms5637_status_i2c_transfer_error
 *
 *********************************************************************/
ms5637_status ms5637_read_prom(void){

	char tx_buf[1];
	char rx_buf[2];
	int byte_count=0;
	int i;

	for(i=0;i<7;i++){
		tx_buf[0] = MS5637_PROM_BASE_ADDR+2*i;
		byte_count = XIic_Send(ms5637_axi_address, MS5637_I2C_ADDR, (u8*)tx_buf, 1, XIIC_STOP);
		if(byte_count!=1){
			return ms5637_status_i2c_transfer_error;
		}

		byte_count = XIic_Recv(ms5637_axi_address, MS5637_I2C_ADDR, (u8*)rx_buf, 2, XIIC_STOP);
		if(byte_count!=2){
			return ms5637_status_i2c_transfer_error;
		}

		ms5637_prom_coeffs[i] = 256*(u32)rx_buf[0] + (u32)rx_buf[1];

	}

	return ms5637_status_ok;

}

/*********************************************************************
 *
 * Function:    ms5637_read_temperature_and_pressure
 *
 * Description:	Use the set oversampling rate to send the proper i2c
 * 				command to start a pressure ADC conversion, then wait
 * 				the appropriate time (based on the OSR) to read the
 * 				raw 24-bit result.  Repeat the same process for
 * 				temperature.  Then compute the compensated temperature
 * 				and pressure values.
 *
 *
 * Parameters:	float* temperature -
 * 					Pointer to temperature variable.  This function
 * 					writes the temperature into this location.
 * 				float* pressure -
 * 					Pointer to temperature variable.  This function
 * 					writes the temperature into this location.
 *
 * Returns:		Enumerated type ms5637_status with possible values
 * 				ms5637_status_ok or ms5637_status_i2c_transfer_error
 *
 *********************************************************************/
ms5637_status ms5637_read_temperature_and_pressure(float* temperature, float* pressure){

	char tx_buf[1];
	char rx_buf[3];
	u8 d1;	// Pressure
	u8 d2;	// Temperature
	int byte_count = 0;
	u32 pres_raw = 0;
	u32 temp_raw = 0;
	int dT;
	int TEMP;
	long long int OFF;
	long long int SENS;
	int P;

	switch(ms5637_resolution){
		case ms5637_resolution_osr_256:
			d1 = MS5637_I2C_CMD_CONVERT_D1_OSR_256;
			break;
		case ms5637_resolution_osr_512:
			d1 = MS5637_I2C_CMD_CONVERT_D1_OSR_512;
			break;
		case ms5637_resolution_osr_1024:
			d1 = MS5637_I2C_CMD_CONVERT_D1_OSR_1024;
			break;
		case ms5637_resolution_osr_2048:
			d1 = MS5637_I2C_CMD_CONVERT_D1_OSR_2048;
			break;
		case ms5637_resolution_osr_4096:
			d1 = MS5637_I2C_CMD_CONVERT_D1_OSR_4096;
			break;
		case ms5637_resolution_osr_8192:
			d1 = MS5637_I2C_CMD_CONVERT_D1_OSR_8192;
			break;
		default:
			d1 = MS5637_I2C_CMD_CONVERT_D1_OSR_8192;
			break;
	}

	tx_buf[0] = d1;
	byte_count = XIic_Send(ms5637_axi_address, MS5637_I2C_ADDR, (u8*)tx_buf, 1, XIIC_STOP);
	if(byte_count!=1){
		return ms5637_status_i2c_transfer_error;
	}

	switch(ms5637_resolution){
		case ms5637_resolution_osr_256:
			usleep(MS5637_CONV_DELAY_OSR_256 * 1000);
			break;
		case ms5637_resolution_osr_512:
			usleep(MS5637_CONV_DELAY_OSR_512 * 1000);
			break;
		case ms5637_resolution_osr_1024:
			usleep(MS5637_CONV_DELAY_OSR_1024 * 1000);
			break;
		case ms5637_resolution_osr_2048:
			usleep(MS5637_CONV_DELAY_OSR_2048 * 1000);
			break;
		case ms5637_resolution_osr_4096:
			usleep(MS5637_CONV_DELAY_OSR_4096 * 1000);
			break;
		case ms5637_resolution_osr_8192:
			usleep(MS5637_CONV_DELAY_OSR_8192 * 1000);
			break;
		default:
			usleep(MS5637_CONV_DELAY_OSR_8192 * 1000);
			break;
	}

	tx_buf[0] = 0x00;
	byte_count = XIic_Send(ms5637_axi_address, MS5637_I2C_ADDR, (u8*)tx_buf, 1, XIIC_STOP);
	if(byte_count!=1){
		return ms5637_status_i2c_transfer_error;
	}

	byte_count = XIic_Recv(ms5637_axi_address, MS5637_I2C_ADDR, (u8*)rx_buf, 3, XIIC_STOP);
	if(byte_count!=3){
		return ms5637_status_i2c_transfer_error;
	}

	pres_raw = (rx_buf[0]<<16) + (rx_buf[1]<<8) + rx_buf[2];


	switch(ms5637_resolution){
		case ms5637_resolution_osr_256:
			d2 = MS5637_I2C_CMD_CONVERT_D2_OSR_256;
			break;
		case ms5637_resolution_osr_512:
			d2 = MS5637_I2C_CMD_CONVERT_D2_OSR_512;
			break;
		case ms5637_resolution_osr_1024:
			d2 = MS5637_I2C_CMD_CONVERT_D2_OSR_1024;
			break;
		case ms5637_resolution_osr_2048:
			d2 = MS5637_I2C_CMD_CONVERT_D2_OSR_2048;
			break;
		case ms5637_resolution_osr_4096:
			d2 = MS5637_I2C_CMD_CONVERT_D2_OSR_4096;
			break;
		case ms5637_resolution_osr_8192:
			d2 = MS5637_I2C_CMD_CONVERT_D2_OSR_8192;
			break;
		default:
			d2 = MS5637_I2C_CMD_CONVERT_D2_OSR_8192;
			break;
	}

	tx_buf[0] = d2;
	byte_count = XIic_Send(ms5637_axi_address, MS5637_I2C_ADDR, (u8*)tx_buf, 1, XIIC_STOP);
	if(byte_count!=1){
		return ms5637_status_i2c_transfer_error;
	}

	switch(ms5637_resolution){
		case ms5637_resolution_osr_256:
			usleep(MS5637_CONV_DELAY_OSR_256 * 1000);
			break;
		case ms5637_resolution_osr_512:
			usleep(MS5637_CONV_DELAY_OSR_512 * 1000);
			break;
		case ms5637_resolution_osr_1024:
			usleep(MS5637_CONV_DELAY_OSR_1024 * 1000);
			break;
		case ms5637_resolution_osr_2048:
			usleep(MS5637_CONV_DELAY_OSR_2048 * 1000);
			break;
		case ms5637_resolution_osr_4096:
			usleep(MS5637_CONV_DELAY_OSR_4096 * 1000);
			break;
		case ms5637_resolution_osr_8192:
			usleep(MS5637_CONV_DELAY_OSR_8192 * 1000);
			break;
		default:
			usleep(MS5637_CONV_DELAY_OSR_8192 * 1000);
			break;
	}

	tx_buf[0] = 0x00;
	byte_count = XIic_Send(ms5637_axi_address, MS5637_I2C_ADDR, (u8*)tx_buf, 1, XIIC_STOP);
	if(byte_count!=1){
		return ms5637_status_i2c_transfer_error;
	}

	byte_count = XIic_Recv(ms5637_axi_address, MS5637_I2C_ADDR, (u8*)rx_buf, 3, XIIC_STOP);
	if(byte_count!=3){
		return ms5637_status_i2c_transfer_error;
	}

	temp_raw = (rx_buf[0]<<16) + (rx_buf[1]<<8) + (rx_buf[2]);

	dT = temp_raw - (((unsigned int)ms5637_prom_coeffs[5])<<8);
	TEMP = 2000 + (unsigned int)((dT * ((long long int)ms5637_prom_coeffs[6]))>>23);

	OFF = (((long long int)ms5637_prom_coeffs[2])<<17) + ((((long long int)ms5637_prom_coeffs[4])*dT)>>6);
	SENS = (((long long int)ms5637_prom_coeffs[1])<<16) + ((((long long int)ms5637_prom_coeffs[3])*dT)>>7);
	P = (int)((((((long long int)pres_raw)*SENS)>>21) - OFF)>>15);

	*pressure = (float)P/100.0f;
	*temperature = (float)TEMP/100.0f;
	
	return ms5637_status_ok;

}

/*********************************************************************
 *
 * Function:    CRC8
 *
 * Description:	Check 1 byte of data with 8 bits of CRC information.
 * 				Use polynomial X^8 + X^5 + X^4 + 1
 * 				For use with TSYS02D serial number read
 *
 * Parameters:	char* data -
 * 					2 byte character array. data[0] is the data byte
 * 					and data[1] contains the CRC information
 *
 * Returns:		TRUE if operation succeeded else FALSE
 *
 *********************************************************************/
int CRC8(char* data){
	u32 div, poly;
	int i;
	div = 256*data[0]+data[1];
	poly = CRC_POLY;
	
	for(i=0;i<8;i++){
		if( (1<<(15-i))&div ){
			div ^= (poly<<(7-i));
		}
	}
	
	if( (div&(0xFF))==0x00 ){
		return TRUE;
	}else{
		return FALSE;
	}
}

/*********************************************************************
 *
 * Function:    CRC16
 *
 * Description:	Check 2 bytes of data with 8 bits of CRC information.
 * 				Use polynomial X^8 + X^5 + X^4 + 1
 * 				For use with TSYS02D serial number read and checking
 * 				ADC results for several sensors
 *
 * Parameters:	char* data -
 * 					3 byte character array. data[0] is the most sig-
 * 					nificant byte, data[1] is the least significant
 * 					byte, and data[2] contains the CRC information
 *
 * Returns:		TRUE if operation succeeded else FALSE
 *
 *********************************************************************/
int CRC16(char* data){
	u32 div, poly;
	int i;
	div = 256*256*data[0]+256*data[1]+data[2];
	poly = CRC_POLY;
	
	for(i=0;i<16;i++){
		if( (1<<(23-i))&div ){
			div ^= (poly<<(15-i));
		}
	}
	
	if( (div&(0xFF))==0x00 ){
		return TRUE;
	}else{
		return FALSE;
	}
}
